%option yylineno
%{

#include <stdio.h>
#include <string.h> /* strncpy */

#define EXTERNAL extern
#include "crstructs.h"
#include "coldread.tab.h"

static int Current_state;

struct sign_table {
  char signBuf[10];  
  int signType;
  size_t signSize;
};


void lex_error(const char *msg);
char *passback_keyword(const char *keyword);

%}

%%

#.*\n { /* Skip comments. */}
^[ \t].*\n { /* More skip comments */ }
[ \t] { /* Ignore spaces too. */ }
\n { /* Skip unaccompanied newlines. */ }

\( { return LBANANA; }
\) { return RBANANA; }

^%[A-z]+%\n {
/* A keyword denoting a list of sentences. */
char *keyword_ptr;
keyword_ptr=passback_keyword(yytext);
strcpy(yylval.wordBuffer,keyword_ptr);
free(keyword_ptr);
Current_state = KEYWD_SENTENCE;
return KEYWORD;
}

![A-z]+! {
/* A keyword which is in a relationship */
  char *keyword_ptr;
  keyword_ptr=passback_keyword(yytext);
  strcpy(yylval.wordBuffer,keyword_ptr);
  free(keyword_ptr);
  Current_state=RELATION_SENTENCE;
  return RELKWD;
}

^[A-z].*\n { 
/* A sentence. */

int pushCurrent_state=Current_state;
size_t yytextLen=strlen(yytext);

Current_state=KEYWD_SENTENCE;
--yytextLen;
yytext[yytextLen] = '\0';
strncpy(yylval.wordBuffer,yytext,sizeof(yylval.wordBuffer)); 
return pushCurrent_state; 
}

[0-9]+ {

/* A digit, in relation to a keyword. Integers only.  */

   yylval.number=atoi(yytext);
   return NUMBER;
}

~ { /* Modulo sign */
   yylval.operation=MOD;
   return COMBSIGN;
}

- { /* A minus sign */
   yylval.operation=SUB;
   return COMBSIGN;
}

\+ { /* A plus sign */
   yylval.operation=ADD;
   return COMBSIGN;
}

\/ { /* Division sign */
   yylval.operation=DIV;
   return COMBSIGN;
}

\* { /* Multiplication */
   yylval.operation=MULT;
   return COMBSIGN;
}

\^ { /* Exponentiation */
   yylval.operation=EXP;
   return COMBSIGN;
}

\= { /* Equality */
   yylval.equalOp=EQ;
   return RELSIGN;
}

\< { /* Less than */
   yylval.equalOp=LT;
   return RELSIGN;
}

> { /* Greater than */
    yylval.equalOp=GT;
    return RELSIGN;
}

!= { /* Not Equal */
   yylval.equalOp=NE;
   return RELSIGN;
}

>= { /* Greater than or equal */
   yylval.equalOp=GE;
   return RELSIGN;
}

\<= { /* Less than or equal */
   yylval.equalOp=LE;
   return RELSIGN;
}

& { /* And */
   yylval.equalOp=AND;
   return RELSIGN;
}

\| { /* Or */
   yylval.equalOp=OR;
   return RELSIGN;
}

&\| { /* Exclusive Or */
   yylval.equalOp=XOR;
   return RELSIGN;
}
. {  /* Mush. */ lex_error("Invalid character"); }

%%
/*
 * Make a keyword for return. Remove extraneous chars from front, back.
 */
char *passback_keyword(const char *keyword)
{
   char *retVal;
   char *workPtr;
   size_t firstSpace;

   ++keyword;
   firstSpace=strcspn(keyword,"\n\r \t");

   retVal=calloc(1,strlen(keyword)+1);

   strcpy(retVal,keyword);

   workPtr = retVal + firstSpace;
   --workPtr; /* Take care of ending keyword indicator */
   *workPtr='\0';

   return retVal;
}

/*
 * Do we have another file to process? Never.
 */
int yywrap(void)
{
  return 1;
}

void lex_error(const char *errMsg)
{
   printf("*** Line %d: %s found at [%s]\n",yylineno,errMsg,yytext);
}
