%{
#include <stdio.h>
#include <string.h> /* strdup */
#define EXTERNAL extern  
#include "crstructs.h"
#include "exprhandlers.h"
#include "relathandlers.h"

extern int yylineno;
extern char *yytext;
%}

%union {
  int number;
  relop equalOp;
  mathop operation;
  relationship *relptr;
  exprInfo * exprptr;
  char wordBuffer[1025];
}

%token <wordBuffer> KEYWORD 
%token <wordBuffer> KEYWD_SENTENCE 
%token <wordBuffer> RELATION_SENTENCE 
%token <wordBuffer> RELKWD 
%token <number> NUMBER 
%token <equalOp> RELSIGN
%token <operation> MATHSIGN
%token <operation> COMBSIGN
%token <operation> LBANANA
%token <operation> RBANANA

%type <exprptr> expression
%type <relptr> relationship

%left MATHSIGN 
%left COMBSIGN


%%

statements: /* Empty. */
| statements statement
;

statement : KEYWD_SENTENCE {make_kwdsentence($1); }
| RELATION_SENTENCE {make_relsentence($1);}
| KEYWORD {make_keyword($1); }
| relationship { /* Already done below */ }
;

expression: 
expression COMBSIGN expression {$$ = mkExprExprExpr($1,$2,$3); } 
| RELKWD COMBSIGN RELKWD {$$ = mkKwdKwdExpr($1,$2,$3);}
| expression COMBSIGN RELKWD {$$ = mkExprKwdExpr($1,$2,$3);}
| RELKWD COMBSIGN expression {$$ = mkKwdExprExpr($1,$2,$3);}
| NUMBER COMBSIGN expression {$$ = mkNumExprExpr($1,$2,$3);}
| expression COMBSIGN NUMBER {$$ = mkExprNumExpr($1,$2,$3);}
| NUMBER COMBSIGN RELKWD {$$ = mkNumKwdExpr($1,$2,$3);}
| RELKWD COMBSIGN NUMBER {$$ = mkKwdNumExpr($1,$2,$3);}
| LBANANA expression RBANANA { $$ = $2; }
;

relationship : 
expression RELSIGN expression {$$ = mkExprExprRel($1,$2,$3);}
| NUMBER RELSIGN expression {$$ = mkNumExprRel($1,$2,$3);}
| expression RELSIGN NUMBER {$$ = mkExprNumRel($1,$2,$3);}
| RELKWD RELSIGN expression {$$ = mkKwdExprRel($1,$2,$3);}
| expression RELSIGN RELKWD {$$ = mkExprKwdRel($1,$2,$3);}
| RELKWD RELSIGN RELKWD {$$ = mkKwdKwdRel($1,$2,$3);}
| RELKWD RELSIGN NUMBER {$$= mkKwdNumRel($1,$2,$3);}
| NUMBER RELSIGN RELKWD {$$ = mkNumKwdRel($1,$2,$3);}
| relationship RELSIGN relationship { 
switch($2)
{
case AND:
case OR:
case XOR:
  $$ = mkRelRelRel($1,$2,$3);
  break;
default:
  yyerror("You can only relate two relationships with a boolean operator");
  YYERROR;
}

}
| LBANANA relationship RBANANA {$$ = $2}
;

%%
/*
 * Put a sentence into the current relationship.
 */
int make_relsentence(char *sentence)
{
  Current_relationship->printoutSentence.sentence_data = strdup(sentence);
  Current_relationship->printoutSentence.sentence_sequence=SentenceSequence;
  ++SentenceSequence;
  return 1;
}

/*
 * Add a sentence to the current keyword's list of sentences.
 */
int make_kwdsentence(char *sentenceWords)
{
  sentence *curSentence=Current_keyword->current_sentence;
  if(! Current_keyword->num_sentences)
  { 
    Current_keyword->sentence_list = (sentence *)calloc(1,sizeof(sentence));
    Current_keyword->current_sentence=Current_keyword->sentence_list;   
    curSentence=Current_keyword->current_sentence;
  }
  else
  {
    curSentence->next_sentence=(sentence *)calloc(1,sizeof(sentence));
    Current_keyword->current_sentence=curSentence->next_sentence;
    curSentence=Current_keyword->current_sentence;
  }

  curSentence->sentence_data=strdup(sentenceWords);
  curSentence->sentence_sequence=SentenceSequence;
  ++SentenceSequence;
  ++Current_keyword->num_sentences;

  return 1;
}

/*
 * Report a grammatical error. 
 */
int yyerror(const char *errMsg)
{
  fprintf(stderr,"%s at line %d\n",errMsg,yylineno);
}


