/*
 * Copyright (C) Charles Shapiro 1997-1999 72300.3632@compuserve.com
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *	
 * Structures and macros for the coldread program
 * Charles Shapiro 4 Jun 1998
 * 
 */
#ifndef CRSTRUCTS_H
#define CRSTRUCTS_H

/*
 *  A member of the list of sentences in a keyword.
 */ 

typedef struct senc
{
  int sentence_sequence;
  char *sentence_data;
  struct senc *next_sentence;
  struct senc *prev_sentence; /* Used only when we sort for output. */
} sentence;

/*
 *  A member of the backbone list of keywords.
 */
typedef struct kwdList {
  int num_sentences;    /* How many sentences are in this structs list */
  int sequence_number;  /* Order this was read in.*/
  int gotten_value;     /* Value from infile (scaled if -c argument given) */
  char *keyword;         /* Key */
  sentence *sentence_list; /* Linked list of sentences */
  sentence *current_sentence; /* Sentence to print? */
  struct kwdList *next_kw;  /* Next member of list of keyword structs.*/
} keylist;
 
typedef enum { EQ=0, NE, LT, GT, LE, GE, AND, OR, XOR  } relop;

typedef enum { ADD=0, SUB, DIV, MULT, EXP, MOD } mathop;

typedef enum { NONUM=0,LHNUM,RHNUM } exprWhichIsNum;

/* 
 * A way of manipulating a pair of  expressions or
 * keywords to come up with a numeric result for input into a relationship.
 */
typedef struct expr {
  keylist *lh_kw_elem;
  keylist *rh_kw_elem;
  mathop evaltype;	
  struct expr *lh_expr_elem;
  struct expr *rh_expr_elem;
  exprWhichIsNum whichNumber;
  int rh_number;
  int lh_number;
  int result_computed; /* Always 0 until computed. */
  int result; /* Computed after all keywords have been read. */
  struct expr *next_expr; 
} exprInfo;

/*
 *  A way of relating a pair of expressions together to determine if a 
 * sentence should be printed.
 */
typedef struct relate {   
  int sequence_number;
  int relation_calculated;
  int shouldPrint;
  sentence printoutSentence;
  exprInfo *lh_memb;
  exprInfo *rh_memb;
  keylist *rh_kw_elem;
  keylist *lh_kw_elem;
  exprWhichIsNum whichNumber;
  int lh_number;
  int rh_number;
  relop relation;
  struct relate *lh_relat;
  struct relate *rh_relat;
  struct relate *next_relat;
} relationship;

EXTERNAL int StoreScaleVal;
EXTERNAL int Sequence;
EXTERNAL keylist *Keyword_pile;
EXTERNAL keylist *Current_keyword;
EXTERNAL exprInfo *Expression_pile;
EXTERNAL exprInfo *Current_expression;
EXTERNAL relationship *Relationship_pile;
EXTERNAL relationship *Current_relationship;
EXTERNAL int SentenceSequence;

keylist *findWordInKeyList(const char *);
sentence *findSentence(int sentenceNumber,sentence *sentenceList);
void initGlobalData(void);
int make_keyword(char *newName);

#define KWTOTAL "_Total_"
#define KWAVERAGE "_Average_"
#define KWNUMBER "_Answers_"

#endif /* CRSTRUCTS_H */
