/*
 * Copyright (C) Charles Shapiro 1997-1999 72300.3632@compuserve.com
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 * Calculate the expressions specified in param file after we have read the
 * data for a reading. 
 *
 * Charles Shapiro 2 Aug 1999
 *
 */
#include <stdio.h> /* sscanf et cetera */

#define EXTERNAL extern
#include "crstructs.h"
#include "coldreader.h"

typedef enum { KEYWORD=2,EXPRESSION=5,NUMBER=9} exprElType; 
#define KWKW 4
#define EXPREXPR 10
#define NUMNUM 18
#define EXPRKW 7
#define NUMKW 11
#define EXPRNUM 14

int calcNumNum(int lhnum,int rhnum,mathop operation);
int calcKwNum(keylist *lhkw,int rhnum,mathop operation);
int calcNumKw(int lhnum,keylist *rhkw,mathop operation);
int calcKwKw(keylist *lhKw,keylist *rhKw,mathop operation);
/* Recursive expressions are handled directly in calls to doXprCalculatio n.*/

/* 
 * So what kind of expression are we looking at here?
 */
exprElType figureType(exprInfo *exprPtr,int lh)
{
  exprElType retVal;

  if(lh)
  {
    if(LHNUM == exprPtr->whichNumber)
      retVal=NUMBER;
    else if(exprPtr->lh_kw_elem)
      retVal = KEYWORD;
    else if(exprPtr->lh_expr_elem)
      retVal= EXPRESSION;
  }
  else
  {
    if(RHNUM == exprPtr->whichNumber)
      retVal=NUMBER;
    else if(exprPtr->rh_kw_elem)
      retVal = KEYWORD;
    else if(exprPtr->rh_expr_elem)
      retVal= EXPRESSION;
  }
  return retVal;
}

/*
 * Calculate the result of an expression.
 */
int calculate_expressions(exprInfo *firstXpr)
{
  exprInfo *curXpr=firstXpr;
  exprElType lhType;
  exprElType rhType;

  while(curXpr) 
  {
    if(! curXpr->result_computed)    
      doXprCalculation(curXpr);    
    curXpr=curXpr->next_expr;
  }
}

/*
 * Calculate the result of a single expression, given the types
 * of the expressions inside it.  This routine recurses.
 */
int doXprCalculation(exprInfo *expression)
{
  int retVal=0;
  int calculation_result;
  exprElType rightType=figureType(expression,0);
  exprElType leftType=figureType(expression,1);
  int callSubType;

  /*
   * Recursion fence.
   */
  if(expression->result_computed)
    return retVal;
  
  callSubType = (int)leftType+(int)rightType;

  switch(callSubType)
  {
  case KWKW:/* Keyword and keyword */
    calculation_result=calcKwKw(expression->rh_kw_elem,
    expression->lh_kw_elem,expression->evaltype);
    break;
  case NUMNUM: /* Number and number */
    calculation_result=calcNumNum(expression->rh_number,
    expression->lh_number,expression->evaltype);
    break;
  case NUMKW: /* Keyword and number */
    if(NUMBER == leftType)
      calculation_result=calcNumKw(expression->lh_number,
      expression->rh_kw_elem,expression->evaltype);
    else
      calculation_result=calcKwNum(expression->lh_kw_elem,
      expression->rh_number,expression->evaltype);
    break;
  case EXPREXPR: /* Two expressions */
    doXprCalculation(expression->lh_expr_elem);
    doXprCalculation(expression->rh_expr_elem);
    calculation_result=calcNumNum((expression->lh_expr_elem)->result,
    (expression->rh_expr_elem)->result,expression->evaltype);
    break;
  case EXPRKW: /* Expression and a keyword */
    if(expression->rh_expr_elem)
    {
      doXprCalculation(expression->rh_expr_elem);
      calculation_result=calcNumKw((expression->lh_expr_elem)->result,
      expression->lh_kw_elem, expression->evaltype);
    }
    else
    {
      doXprCalculation(expression->lh_expr_elem);
      calculation_result=calcKwNum(expression->lh_kw_elem,
      (expression->rh_expr_elem)->result,
      expression->evaltype);
    }
    break;
  case EXPRNUM: /* Expression and a number */
    if(LHNUM == expression->whichNumber)
    {
      doXprCalculation(expression->rh_expr_elem);
      calculation_result=calcNumNum(expression->lh_number,
      (expression->rh_expr_elem)->result,expression->evaltype);
    }
    else
    {
      doXprCalculation(expression->lh_expr_elem);
      calculation_result=calcNumNum((expression->lh_expr_elem)->result,
      expression->rh_number,expression->evaltype);
    }
    break;
  default:
    app_error_message(
    "Uh oh. Tried to parse an expression from a keyword not yet read.",NULL);
    retVal=-1;
    break;
  }

  expression->result_computed=1;
  expression->result=calculation_result;
  return retVal;
}

/*
 * Calculate the result of an expression consisting of a pair of numbers.
 * This is used in recursive calls to expressions as well as in its 
 * (admittedly less useful) function in calculating expressions such as
 * "(4+3)".
 */
int calcNumNum(int lhnum,int rhnum,mathop operation)
{
  int retVal;

  switch(operation)
  {
  case ADD:
    retVal=lhnum+rhnum;
    break;
  case SUB:
    retVal=lhnum-rhnum;
    break;
  case DIV:
    if(rhnum)
      retVal=lhnum / rhnum;
    else
      retVal=0;
    break;
  case MULT:
    retVal=lhnum * rhnum;
    break;
  case EXP:
    retVal=lhnum;
    while(rhnum)
    {
      --rhnum;
      if(rhnum)
	retVal *= lhnum;
    }
    break;
  case MOD:
    if(rhnum)
      retVal = lhnum % rhnum;
    else
      retVal=rhnum;
    break;
  }

  return retVal;
}
/*
 * Calculate an expression consisting of a keyword and a number.
 */
int calcKwNum(keylist *lhkw, int rhnum, mathop operation)
{
  int retVal;

  retVal = calcNumNum(lhkw->gotten_value,rhnum,operation);

  return retVal;

}
/*
 *  Calculate an expression consisting of a number and a keyword.
 */
int calcNumKw(int lhnum, keylist *rhkw,mathop operation)
{
  int retVal;
  retVal=calcNumNum(lhnum,rhkw->gotten_value,operation);
  return retVal;
}
/*
 * Calculate an expression consisting of a pair of keywords.
 */
int calcKwKw(keylist *lhkw,keylist *rhkw,mathop operation)
{
  int retVal;
  
  retVal=calcNumNum(lhkw->gotten_value,rhkw->gotten_value,operation);

  return retVal;
}
