/*
 * Copyright (C) Charles Shapiro 1997-1999 72300.3632@compuserve.com
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 * Handle various types of expressions passed from coldread.y
 * 
 * Charles Shapiro 3 Jun 1999
 *
 */
#include <stdio.h>
#define EXTERNAL extern
#include "crstructs.h"

/*
 * Make a new place to put an expression.
 */
exprInfo *makeNewExpressionOnPile(mathop opForPile)
{
  if(! Expression_pile)  
  {
    Expression_pile = (exprInfo *)calloc(1,sizeof(exprInfo));
    Current_expression = Expression_pile;
  }
  else
  {
    Current_expression->next_expr=(exprInfo *)calloc(1,sizeof(exprInfo));
    Current_expression = Current_expression->next_expr;
  }
  Current_expression->evaltype=opForPile;
  return Current_expression;
}

/*
 * Make an expression from a pair of expressions and an arithmetic sign
 * e.g. (!dog! + 4) * (!cat! - 3)
 */
exprInfo *mkExprExprExpr(exprInfo * lhExpr,mathop middle, exprInfo * rhExpr)
{
  exprInfo *newExpression;
  newExpression = makeNewExpressionOnPile(middle);
  newExpression->evaltype=middle;

  newExpression->lh_expr_elem=lhExpr;
  newExpression ->rh_expr_elem=rhExpr;

  return newExpression;
}
/*
 * Make an expression from a pair of keywords and a math sign
 * e.g. !dog! + !cat!
 */ 
exprInfo *mkKwdKwdExpr(char *lhKwd,mathop middle, char *rhKwd)
{
  exprInfo *newExpression=makeNewExpressionOnPile(middle);
  keylist *keyListPtr;

  keyListPtr=findWordInKeyList(lhKwd);
  newExpression->lh_kw_elem=keyListPtr;
  keyListPtr=findWordInKeyList(rhKwd);
  newExpression->rh_kw_elem=keyListPtr;

  return newExpression;
}
/*
 * Make an expression from an expression and a keyword.
 * e.g. (!dog! - 4) + !cat!
 */
exprInfo *mkExprKwdExpr(exprInfo * lhExpr,mathop middle,char *rhKwd)
{
  exprInfo *newExpression=makeNewExpressionOnPile(middle);

  newExpression->lh_expr_elem=lhExpr;
  newExpression->rh_kw_elem=findWordInKeyList(rhKwd);

  return newExpression;
}
/*
 * Make an expression from a keyword, a math sign, and an expression
 * e.g. !cat! / (!dog! - !rat!)
 */
exprInfo *mkKwdExprExpr(char *lhKwd,mathop middle,exprInfo * rhExpr)
{
  exprInfo *newExpression=makeNewExpressionOnPile(middle);

  newExpression->rh_expr_elem=rhExpr;
  newExpression->lh_kw_elem=findWordInKeyList(lhKwd);

  return newExpression;
}
/*
 * Make an expression from an expression and a number
 * e.g. 3 * (!dog! + 4)
 */
exprInfo *mkNumExprExpr(int lhNum,mathop middle,exprInfo * rhExpr)
{
  exprInfo *newExpression=makeNewExpressionOnPile(middle);

  newExpression->lh_number=lhNum;
  newExpression->whichNumber=LHNUM;
  newExpression->rh_expr_elem=rhExpr;

  return newExpression;
}
/*
 * Make an expression from an expression, an operation, and a number
 * e.g. (!dog! + 3) / 4
 */
exprInfo *mkExprNumExpr(exprInfo *lhExpr,mathop middle, int rhNum)
{
  exprInfo *newExpression=makeNewExpressionOnPile(middle);

  newExpression->rh_number=rhNum;
  newExpression->whichNumber=RHNUM;
  newExpression->lh_expr_elem=lhExpr;

  return newExpression;
}
/*
 * Make an expression from a number, a mathsign, and a keyword
 * e.g. 2 - !dog!
 */
exprInfo *mkNumKwdExpr(int lhNum,mathop middle,char *rhKwd)
{
  exprInfo *newExpression=makeNewExpressionOnPile(middle);
  
  newExpression->lh_number=lhNum;
  newExpression->whichNumber=LHNUM;
  newExpression->rh_kw_elem=findWordInKeyList(rhKwd);

  return newExpression;
}
/*
 * Make an expression from a keyword, a math operator, and a number
 * e.g. !dog! / 4
 */
exprInfo *mkKwdNumExpr(char *lhKwd,mathop middle, int rhNum)
{
  exprInfo *newExpression=makeNewExpressionOnPile(middle);

  newExpression->lh_kw_elem=findWordInKeyList(lhKwd);
  newExpression->rh_number=rhNum;
  newExpression->whichNumber=RHNUM;

  return newExpression;
}





