/*
 *   Copyright (C) Charles Shapiro 1997-1999 72300.3632@compuserve.com
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 * Perform a cold reading. Scale values appropriately.
 */
#include <stdio.h>
#include <string.h>

#define EXTERNAL extern
#include "crstructs.h"


/*
 * Calculate real value for keyword struct, given number of sentences and
 * scaling factors.
 */
int scale_coldval(int numSentences,int inValue, int inBottom, int inTop)
{
  int retVal;
  int scaleRange=inTop - inBottom;
  
  if(numSentences) /* Handle keywords which contain no sentences. */
  {
    if(scaleRange == numSentences)     
      retVal=inValue;

    if(scaleRange > numSentences)
      retVal=scaleNStoVal(numSentences,inValue,scaleRange);

    if(scaleRange < numSentences)
      retVal=scaleValtoNS(numSentences,inValue,scaleRange);
  }

  return retVal;

 }

/*
 * Calculate real value for keyword struct, given number of sentences and
 * scaling factors, assuming we have more sentences than possible values.
 */
 int scaleValtoNS(int numSentences,int inValue,int scaleRange)
{
  
  int retVal=-1;
  int modRetVal;
  int rulerValue=numSentences / scaleRange;
  
  retVal=inValue * rulerValue;

  modRetVal = retVal % scaleRange;

  if(modRetVal > (rulerValue / 2) && 
  (retVal < numSentences))
    ++retVal;

  return retVal;
}
/*
 * Fill auto-keywords out.
 */
int make_autoKeywords(int grandTotal,int numAnswers)
{
  keylist *kwdPtr;  
  int numAverage=0;

  if(numAnswers)
    numAverage = grandTotal / numAnswers;

  kwdPtr=findWordInKeyList(KWTOTAL);
  kwdPtr->gotten_value=grandTotal;
  kwdPtr=findWordInKeyList(KWAVERAGE);
  kwdPtr->gotten_value=numAverage;
  kwdPtr=findWordInKeyList(KWNUMBER);
  kwdPtr->gotten_value=numAnswers;
  
}
/*
 * Calculate real value for keyword struct, given number of sentences and
 * scaling factors, assuming we have more possible values than sentences.
 */
int scaleNStoVal(int numSentences,int inValue,int scaleRange)
{

  int retVal=-1;
  int modRetval;

  int rulerValue=scaleRange / numSentences;

  int modRuler=scaleRange % numSentences;

  retVal= inValue / rulerValue;

  modRetval= inValue % rulerValue;

  if((modRetval > modRuler) && 
  (retVal < numSentences))
    ++retVal;

  return retVal;  
}

/*
 *  Read cold reading data; update appropriate member of keyword list.
 *  A Real Man would write another yypars e() routine for this, but
 *  that will probably wait for the next version.
 */
int do_reading(FILE *infileP)
{
  char inputLine[BUFSIZ]; /* Should not need near this much. */
  char inKeyword[BUFSIZ]; /* Ditto. */
  int inKeyVal;
  int inKeyBottom;
  int inKeyTop;
  int scaled_inputval;
  int status=0;
  keylist *curKeylistP;
  int numValues=0;
  int valTotal=0;


  memset(inputLine,0,sizeof(inputLine));

  while( (fgets(inputLine,(int)(sizeof(inputLine)-1),infileP)) && 
  (0 == status) )
  {
    if(0 != parseKeyLine(inputLine,
    inKeyword,&inKeyVal,&inKeyBottom,&inKeyTop))
    {
      app_error_message("Error: malformed input line:",inputLine);
      status=-1;
    }
    if(! status)  
    {  
      ++numValues;    
      valTotal += inKeyVal;
    }
    if(curKeylistP = findWordInKeyList(inKeyword))
    {
      curKeylistP->gotten_value=inKeyVal;
      scaled_inputval=scale_coldval(curKeylistP->num_sentences,
      inKeyVal,inKeyBottom,inKeyTop);
      curKeylistP->current_sentence=findSentence(scaled_inputval,
      curKeylistP->sentence_list);
      if(StoreScaleVal)
	curKeylistP->gotten_value=scaled_inputval;
    }
    else
      app_error_message("Warning: Cannot find keyword in parms file:",
      inKeyword);
    
  }

  make_autoKeywords(valTotal,numValues);
  return status;
}
