/*
 * Copyright (C) Charles Shapiro 1997-1999 72300.3632@compuserve.com
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *  Gather the appropriate data from the tree(s) in memory, sort it, and
 *  print it out.
 * 
 *  Charles Shapiro 18 Sep 1999
 *
 */
#include <stdio.h>
#include <stdlib.h> /* calloc,qsort */
#include <string.h> /* strdup */

#define EXTERNAL extern
#include "crstructs.h"
#include "coldreader.h"

/*
 * Structure used to sort sentences
 */
typedef struct sort_S {
  sentence *sp;
  int sNum;
} sentenceArrayMemb;

/*
 * Construct another member of the output sentence list.
 */
sentence *addSentenceToList(int foundFirst,
sentence *inList,const char *sentenceWords,
int sequence)
{  
  sentence *newSentence;

  if(foundFirst)
    newSentence=(sentence *)calloc(1,sizeof(sentence));
  else
    newSentence=inList;

  if(newSentence)
  {
    newSentence->sentence_data=strdup(sentenceWords);
    newSentence->sentence_sequence=sequence;
    if(foundFirst)
      newSentence->prev_sentence=inList;
  }
  if(foundFirst)
    inList->next_sentence=newSentence;

  return newSentence;
}

/*
 * Get a list of the actual sentences in a cold reading.
 */
int calculate_reading(sentence *outputList, const keylist *keywordP, 
const relationship *relatP)
{
  const relationship *relation_cursor=relatP;
  const keylist *keyword_cursor=keywordP;
  int numSentences=0;

  while(relation_cursor)
  {
    /* 
     * Do not try to print if relation is not true or if it has no
     * sentence to print (e.g. it is part of a boolean relationship)
     */
    if((relation_cursor->shouldPrint) &&  
    (relation_cursor->printoutSentence.sentence_data))
    {
      outputList=addSentenceToList(numSentences,outputList,
      relation_cursor->printoutSentence.sentence_data,
      relation_cursor->printoutSentence.sentence_sequence);
      ++numSentences;
    }
    relation_cursor=(relationship *)relation_cursor->next_relat;
  }

  while(keyword_cursor)
  {
    
    sentence *output_sentence=0;
    /* Do not try to get sentences from keywords which have none */
    if(keyword_cursor->num_sentences) 
      output_sentence = keyword_cursor->current_sentence; 

    if(output_sentence)
    {
      outputList=addSentenceToList(numSentences,
      outputList,output_sentence->sentence_data,
      output_sentence->sentence_sequence);
      ++numSentences;
    }
    keyword_cursor=(keylist *)keyword_cursor->next_kw;
  }
  return numSentences;
}

/*
 * Comparision function for sort of sentence data.
 */
int compareSequence(const void *sentence1,const void *sentence2)
{
  int retVal;
  const sentenceArrayMemb *first=(sentenceArrayMemb *)sentence1;
  const sentenceArrayMemb *second=(sentenceArrayMemb *)sentence2;

  retVal=(first->sNum) - (second->sNum);
  
  return retVal;
}

/*
 * Sort the reading sentences we have gathered by sequence number.
 * Takes the _address_of_ the linked list of sentences we have gathered.
 */
int sort_reading(sentence **readingPtr,int numSentences)
{
  sentence *listPtr=*readingPtr;
  sentenceArrayMemb *sentenceArray=
  (sentenceArrayMemb *)calloc(numSentences,sizeof(sentenceArrayMemb));
  sentence *listCursor=listPtr;
  int ii;
  
  for(ii=0;ii<numSentences;++ii)
  {
    sentenceArray[ii].sp=listCursor;
    sentenceArray[ii].sNum=listCursor->sentence_sequence;
    listCursor=listCursor->next_sentence;
  }
  qsort((void *)sentenceArray,numSentences,sizeof(sentenceArrayMemb),
  compareSequence);

  listCursor=sentenceArray[0].sp;  
  listCursor->prev_sentence=0;
  listCursor->next_sentence=sentenceArray[1].sp;
  *readingPtr=listCursor;

  for(ii=1;ii<numSentences;++ii)
  {
    listCursor=sentenceArray[ii].sp;
    listCursor->prev_sentence=sentenceArray[ii-1].sp;
    if(ii<(numSentences-1))      
      listCursor->next_sentence=sentenceArray[ii+1].sp;
    else
      listCursor->next_sentence=0;
  }
  return 0;
}
/*
 * Print out our cold reading.
 */
int print_reading(const sentence *reading,FILE *outFile)
{

  const sentence *sentenceCursor=reading;

  while(sentenceCursor)
  {
    fprintf(outFile,"%s\n",sentenceCursor->sentence_data);
    sentenceCursor=sentenceCursor->next_sentence;
  }
  return;
}
