/*
 *   Copyright (C) Charles Shapiro 1997-1999 72300.3632@compuserve.com
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 * Handle relationships between pairs of expressions, numbers, and keywords.
 *
 * Charles Shapiro 4 Jun 1999
 */
#include <stdio.h>
#define EXTERNAL extern
#include "crstructs.h"

relationship *addNewRelationshipToPile(relop relateOp)
{
  if(! Relationship_pile)
  {
    Relationship_pile = (relationship *)calloc(1,sizeof(relationship));
    Current_relationship = Relationship_pile;
  }
  else
  {
    Current_relationship->next_relat=
    (relationship *)calloc(1,sizeof(relationship));
    Current_relationship = Current_relationship->next_relat;
  }
  Current_relationship->relation=relateOp;
  Current_relationship->sequence_number=Sequence;
  ++Sequence;
  return Current_relationship;
}
/*
 * Relate a pair of expressions:
 * !cat! + 12 > !dog! - 3
 */
relationship *mkExprExprRel(exprInfo * lhExpr,relop middle,exprInfo * rhExpr)
{
  relationship *newRelation=addNewRelationshipToPile(middle);

  newRelation->lh_memb=lhExpr;
  newRelation->rh_memb=rhExpr;

  return newRelation;
}
/*
 * Add a relationship between a number and an expression:
 * 17 = !dog! + 5 
 */
relationship * mkNumExprRel(int lhNum,relop middle,exprInfo *rhExpr)
{
  relationship *newRelation=addNewRelationshipToPile(middle);

  newRelation->rh_memb=rhExpr;
  newRelation->lh_number=lhNum;
  newRelation->whichNumber=LHNUM;

  return newRelation;
}
/*
 * Make a relationship between an expression and a number:
 * !dog! + 5 = 17
 */
relationship *mkExprNumRel(exprInfo * lhExpr,relop middle,int rhNum)
{
  relationship *newRelation=addNewRelationshipToPile(middle);  

  newRelation->lh_memb=lhExpr;
  newRelation->rh_number=rhNum;
  newRelation->whichNumber=RHNUM;
  return newRelation;
}
/*
 * Handle a relationship between a keyword and an expression:
 * !dog! > !cat!+5
 */
relationship *mkKwdExprRel(const char *lhKwd,relop middle,exprInfo * rhExpr)
{
  relationship *newRelation=addNewRelationshipToPile(middle);

  newRelation->rh_memb=rhExpr;
  newRelation->lh_kw_elem=findWordInKeyList(lhKwd);

  return newRelation;
}
/*
 * Handle relationship between an expression and a keyword:
 * !dog!+5 > !cat
 */
relationship *mkExprKwdRel(exprInfo *lhExpr,relop middle,const char *rhKwd)
{
  relationship *newRelation=addNewRelationshipToPile(middle);
  
  newRelation->lh_kw_elem=findWordInKeyList(rhKwd);
  newRelation->lh_memb=lhExpr;

  return newRelation;
}
/*
 * Handle a relationship between a pair of keywords:
 * !fox! > !dog!
 */
relationship *mkKwdKwdRel(const char *lhKwd,relop middle, const char *rhKwd)
{
  relationship *newRelation=addNewRelationshipToPile(middle);

  newRelation->lh_kw_elem=findWordInKeyList(lhKwd);
  newRelation->rh_kw_elem=findWordInKeyList(rhKwd);

  return newRelation;
}
/*
 * Handle relationship between keyword and number
 * e.g. !dog! > 5
 */
relationship *mkKwdNumRel(const char *lhKwd,relop middle,int rhNumber)
{
  relationship *newRelation=addNewRelationshipToPile(middle);

  newRelation->lh_kw_elem=findWordInKeyList(lhKwd);
  newRelation->rh_number=rhNumber;
  newRelation->whichNumber=RHNUM;

  return newRelation;
}
/*
 * Handle a relationship between a number and a keyword:
 * 4 > !dog!
 */
relationship *mkNumKwdRel(int lhNum,relop middle,const char *rhKwd)
{
  relationship *newRelation=addNewRelationshipToPile(middle);
  newRelation->rh_kw_elem=findWordInKeyList(rhKwd);
  newRelation->lh_number=lhNum;
  newRelation->whichNumber=LHNUM;

  return newRelation;

}

/*
 * Handle a relationship between a pair of relationships. This is 
 * really designed for logical operators.
 */
relationship *mkRelRelRel(relationship *lhMemb,relop middle, 
relationship *rhMemb)
{
  relationship *newRelation=addNewRelationshipToPile(middle);
  
  newRelation->lh_relat=lhMemb;
  newRelation->rh_relat=rhMemb;
 
  return newRelation;
}
