/*
 *   Copyright (C) Charles Shapiro 1997-1999 72300.3632@compuserve.com
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 * Figure out what we will output after reading our param file, and our reading
 * data, and after calculating the values of all expressions.
 *
 * Charles Shapiro 6 Sep 1999
 *
 */
#include <stdio.h>
#define EXTERNAL extern
#include "crstructs.h"
#include "coldreader.h"

typedef enum {KW=1,INT=2,EXPR=5} element_type;
#define KWINT 3
#define EXPRINT 7
#define EXPRKW 6


/*
 * Find the value in an expression, keyword, or int.
 */
int retrieve_value(void *thing,element_type thingType)
{
  int retVal;
  keylist *keyl_ptr;
  exprInfo *express_ptr;
  int *int_ptr;

  switch(thingType)
  {
  case KW:
    keyl_ptr=(keylist *)thing;
    retVal=keyl_ptr->gotten_value;
    break;
  case EXPR:
    express_ptr=(exprInfo *)thing;
    retVal=express_ptr->result;
    break;
  default:
    int_ptr=(int *)thing;
    retVal=*int_ptr;
    break;
  }
  return retVal;
}

/*
 * Find the value of a member of a relationship.
 */
int findMembval(relationship *relp,int rhmember)
{
  int retVal;

  if(rhmember)
  {
    if(relp->rh_memb)
      retVal=retrieve_value(relp->rh_memb,EXPR);
    else if (relp->rh_kw_elem)
      retVal=retrieve_value(relp->rh_kw_elem,KW);
    else if(RHNUM == relp->whichNumber)
      retVal=relp->rh_number;
  }
  else 
  {
    if(relp->lh_memb)
      retVal=retrieve_value(relp->lh_memb,EXPR);
    else if (relp->lh_kw_elem)
      retVal=retrieve_value(relp->lh_kw_elem,KW);
    else if(LHNUM == relp->whichNumber)
      retVal=relp->lh_number;
  }

  return retVal;
}
/*
 * Deal with logical operations
 */
int handle_logicRel(relationship *relPtr)
{
  int shouldLeft=0;
  int shouldRight=0;
  int should_print=0;

  shouldLeft=calcRelationship(relPtr->lh_relat);
  shouldRight=calcRelationship(relPtr->rh_relat);
  switch(relPtr->relation)
  {
  case OR:
    if(shouldLeft || shouldRight)
      should_print=1;
    break;
  case AND:
    if(shouldLeft && shouldRight)
      should_print=1;
    break;
  case XOR:
    if((shouldLeft || shouldRight) && (! (shouldLeft && shouldRight)) )
      should_print=1;
    break;
  }
  relPtr->shouldPrint=should_print;
  relPtr->relation_calculated=1;
  return should_print;

}

/*
 * Should we print out the sentence below this relationship?
 * This may be called recursively for relationships paired by a boolean.
 */
int calcRelationship(relationship * relationP) 
{

  int lhSide;
  int rhSide;
  int should_print=0;

  if(! relationP->relation_calculated)
  {
    lhSide=findMembval(relationP,0);
    rhSide=findMembval(relationP,1);
    switch(relationP->relation)
    {
    case EQ:
      if(lhSide == rhSide)
	should_print = 1;
      break;
    case NE:
      if(lhSide != rhSide)
	should_print = 1;
      break;
    case LT:
      if(lhSide < rhSide)
	should_print = 1;
      break;
    case GT:
      if(lhSide > rhSide)
	should_print=1;
      break;
    case LE:
      if(lhSide <= rhSide)
      should_print=1;
      break;
    case GE:
      if(lhSide >= rhSide)
	should_print=1;
      break;
    case AND:
    case OR:
    case XOR:
      should_print = handle_logicRel(relationP);
      break;
    }  
    relationP->shouldPrint=should_print;
    relationP->relation_calculated=1;
  }
  else
    should_print=relationP->shouldPrint;

  return should_print;
}
/*
 * Calculate the relationships to determine which sentence gets
 * printed.  The main line for this file.
 */
int calculate_relationships(relationship *relationPtr)
{
  int retVal=0;

  while(relationPtr)
  {
    calcRelationship(relationPtr);
    relationPtr=(relationship *)relationPtr->next_relat;
  }
  return retVal;
}
